import json
import sys
import time
import getopt

from json.decoder import JSONDecodeError
from datetime import datetime
from itertools import count
from multiprocessing import Process
from epever_control import EpeverChargeController

config_file =""
set_date =""
argv = sys.argv[1:]
try:
    options, args = getopt.getopt(argv, "c:d:",
                               ["config_file =",
                                "set_date ="])
except:
    print("Usage: epever-charge-controller.py <config-file>")
    sys.exit()
 
for name, value in options:
    if name in ['-c', '--config_file']:
        config_file = value
    elif name in ['-d', '--set_date']:
        set_date = value
 
# read configuration file
try:
    with open(config_file, 'r') as myfile:
        data=myfile.read()
except FileNotFoundError:
    print("Configuration file <"+ config_file +"> not found.")
    sys.exit();

if set_date:
    print("Setting the time and date.")
    sys.exit()
else:
    # parse configuration file
    try:
        obj = json.loads(data)
        tmp = obj['epever_data_file']
        tmp1 = obj['RS485_port']
    except JSONDecodeError as e:
        print("Error decoding configuration file. Not JSON format?")
        sys.exit()
    except TypeError as e:
        print("Error decoding configuration file: JSON objects not str, bytes or bytearray.")
    except KeyError:
        print("Error reading configuration file: needed fields not found.")
        sys.exit()

    # show values from config file
    print("Using RRS485 port: " + str(obj['RS485_port']))
    print("Using RRS485 address: " + str(obj['RS485_address']))
    print("Saving EPEVER data to: " + str(obj['epever_data_file']))

    #controller = EpeverChargeController(obj['RS485_port'], 1)
    controller = EpeverChargeController(obj['RS485_port'], obj['RS485_address'])

    rawdat={}
    rawdat['solar_voltage'] = controller.get_solar_voltage()
    rawdat['solar_current'] = controller.get_solar_current()
    rawdat['solar_power'] = controller.get_solar_power_l() + 256*controller.get_solar_power_h()

    rawdat['load_voltage'] = controller.get_load_voltage()
    rawdat['load_current'] = controller.get_load_current()
    rawdat['load_power'] = controller.get_load_power_l() + 256*controller.get_load_power_h()

    rawdat['battery_voltage'] = controller.get_battery_voltage()
    rawdat['battery_current'] = controller.get_battery_current()
    rawdat['battery_power'] = controller.get_battery_power_l() + 256*controller.get_battery_power_h()

    rawdat['controller_temp'] = controller.get_controller_temperature()
    rawdat['ambient_temp'] = controller.get_ambient_temperature()

    rawdat['generated_energy_today'] = controller.get_generated_energy_today_l() + 256*controller.get_generated_energy_today_h()

    rawdat['date_time'] = controller.get_datetime()

    rawdat['battery_status'] = controller.get_battery_status()
    rawdat['controller_status'] = controller.get_charging_equipment_status()

    print (json.dumps(rawdat, indent=1, sort_keys=False))

    datetime={}    
    datetime['year'] = 21
    datetime['month'] = 11
    datetime['day'] = 12
    datetime['hours'] = 15
    datetime['minutes'] = 50
    datetime['seconds'] = 0

    controller.write_date(datetime)

    #controller.switch_load_on()
