# Copyright 2021 Marcel Konstapel
# https://www.meezenest.nl/mees
#
# This file is part of SolarGeneratorDatalogger.
# A set of programs and scripts to log all the available data from a self build solar generator.
# It runs on a Raspberry Pi Zero 1 W with Raspbian
#
# It logs data from a EPEVER Soalrge Charge Controller (via Bluetooth) and an EPEVER solar charger (via RS485-USB).
# The data is analyzed and stored in a file. The data is also available via a web browser over Wifi
# (Raspberry Pi acts as access point).
# Future plan is to implement LoRa so the data can be accessed remotely.
#
# This program processes the csv file from the previous day
# It calculates various values and appends this as a row to the big_data file
#
# Usage of program:
#
#  process_daily_data config.json
#
# V0.1 2021-11-15
#   - First working program
#
#    SolarGeneratorDatalogger is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SolarGeneratorDatalogger is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SolarGeneratorDatalogger.  If not, see <https://www.gnu.org/licenses/>.
#
import json
import sys
import time
import getopt
import csv
import os.path
import pandas as pd

from json.decoder import JSONDecodeError
from datetime import datetime

if (len(sys.argv)<2):
    print("Usage: get_used_capacity.py <config-file>")
else:

    # read configuration file
    try:
        with open(sys.argv[1], 'r') as myfile:
            data=myfile.read()
    except FileNotFoundError:
        print("Configuration file <"+ sys.argv[1] +"> not found.")
        sys.exit();

    # parse configuration file
    try:
        obj = json.loads(data)
        tmp = obj['csv_bms_data_file']
    except JSONDecodeError as e:
        print("Error decoding configuration file. Not JSON format?")
        sys.exit()
    except TypeError as e:
        print("Error decoding configuration file: JSON objects not str, bytes or bytearray.")
    except KeyError:
        print("Error reading configuration file: needed fields not found.")
        sys.exit()

    # show values from config file
    #print("Reaidng BMS csv data from    : " + str(obj['csv_bms_data_file']))
    
    # open BMS data file and extract values
    try:
        data = pd.read_csv(obj['csv_bms_data_file'])
    except FileNotFoundError:
        print("Input file <"+ obj['csv_bms_data_file'] +"> not found.")
        sys.exit();

    # Get delta of energy stored
    result = data.iloc[-1]['remain_cap'] - data.iloc[0]['remain_cap']
    
    print(round(result,4))
