#!/bin/bash

# Main loop of SolarGenerator
# This loop reads the values from the battery as well as the solar charger
# When the battery capacity is to low, the SolarGenerator is switched off.
# When the battery is charged up again, the SolarGenerator is switched on again.

battery_switch_on_value=10
battery_switch_off_value=5

current_date=$(/usr/bin/date +'%d-%m-%Y')
previous_date=$(/usr/bin/date +'%d-%m-%Y')

# Get file names of data logging files
json_bms_file=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/config.json -f liontron_bms_data_file)
json_solar_file=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/config.json -f epever_data_file)

loop_counter=0

while :
do
    # Get the values from the battery
    /usr/bin/python /home/marcel/SolarGenerator/liontron-bms-reader.py /home/marcel/SolarGenerator/config.json

    # Get the values from the solar generator
    /usr/bin/python /home/marcel/SolarGenerator/epever-charge-controller.py  -c /home/marcel/SolarGenerator/config.json
    
    # Plot diagrams for web site
    /home/marcel/SolarGenerator/plot_diagrams.sh
    
    # copy data files to web site
    cp $json_bms_file /var/www/html/
    cp $json_solar_file /var/www/html/

    # Get the remaining capacity from the battery
    remain_cap_percent=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/liontron_bms_data.json -f remain_cap_percent)

    # Is the SolarGenerator switched on or off?
    load_on=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/epever_data.json -f load_on)

    # Switch on load when battery is has at least $battery_switch_on_value% charge
    if [ $remain_cap_percent -gt $battery_switch_on_value ]
    then

        if [ $load_on = 'False' ]
        then
            /usr/bin/python /home/marcel/SolarGenerator/epever-charge-controller.py -c /home/marcel/SolarGenerator/config.json -l on
        fi

        echo "Battery sufficiently charged ($remain_cap_percent%). Load is switched on."

    # Switch off load when battery has less charge than battery_switch_off_value%
    elif [ $remain_cap_percent -lt $battery_switch_off_value ]
    then

        if [ $load_on = 'True' ]
        then
            /usr/bin/python /home/marcel/SolarGenerator/epever-charge-controller.py -c /home/marcel/SolarGenerator/config.json -l off
        fi

        echo "Battery low ($remain_cap_percent%). Load is switched off."

    # Hysteresis
    else

        if [ $load_on = 'True' ]
        then
            echo "Battery almost empty ($remain_cap_percent%). Load is still switched on."
        else
            echo "Battery almost empty ($remain_cap_percent%). Load is still switched off."
        fi
    fi

    echo "The SolarGenerator is switched off when the SoC is less than $battery_switch_off_value%."
    echo "The SolarGenerator is switched on when the SoC is greater than $battery_switch_on_value%."

    current_date=$(/usr/bin/date +'%d-%m-%Y')
    # Append new values to existing file
    if [ $current_date = $previous_date ]
    then

       echo "Still the same day"

       # Now let's log some data to the csv files from this day:
       /usr/bin/python /home/marcel/SolarGenerator/log_data_to_csv.py /home/marcel/SolarGenerator/config.json


    # It is the next day, so make new data file
    else
       echo "The next day"

       # Process data from the csv files from the previous day
       # and store it in the a log file

       # Get file names of data logging files
       csv_bms_data_file=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/config.json -f csv_bms_data_file)
       csv_solar_data_file=$(/usr/bin/python /home/marcel/SolarGenerator/get_field_from_json.py -i /home/marcel/SolarGenerator/config.json -f csv_solar_data_file)

       # Process data from this day:
       /usr/bin/python /home/marcel/SolarGenerator/process_daily_data.py /home/marcel/SolarGenerator/config.json

       # Make archive from latest data files and make it available via the website
       /usr/bin/tar -czf /var/www/html/solar-generator-data.tar.gz /home/marcel/SolarGenerator/*.csv

       # Remove old data files
       rm $csv_bms_data_file
       rm $csv_solar_data_file

       # Now let's log some data from this day to the new csv files:
       /usr/bin/python /home/marcel/SolarGenerator/log_data_to_csv.py /home/marcel/SolarGenerator/config.json
       
    fi

    previous_date=$current_date
    
    # Calculate energy consumption of battery every 15 minutes or so
    if [ $loop_counter -gt 14 ]
    then
        /usr/bin/python /home/marcel/SolarGenerator/get_used_capacity.py /home/marcel/SolarGenerator/config.json > /var/www/html/delta_energy.txt
        let "loop_counter = 0"
    else
       let "loop_counter += 1"
    fi
    
    echo
    echo "Loop repeats every 60 seconds. Press [CTRL+C] to stop.."
    sleep 60

done
