var dp = {
  // (A) CREATE TIME PICKER
  instances : [], // All time picker instances
  init: function (wrapper, target) {
  // wrapper - container to generate time picker into
  // target - optional, target input field for inline time pickers

    // (A1) CREATE NEW INSTANCE + "GET ID"
    let id = dp.instances.length;
    dp.instances.push({ wrap : wrapper });
    let inst = dp.instances[id];
    if (target != undefined) { inst.target = target; }

    // (A2) TIME PICKER ITSELF
    let picker = document.createElement("div");
    picker.className = "dp";
    inst.wrap.appendChild(picker);

    // (A3) *THE* BUTTONATOR - HR + MIN
    let buttonator = function (segment) {
      // Button Container
      let box = document.createElement("div");
      box.className = "dp-box";
      
      // Current Value
      let description = document.createElement("input");
      description.type = "text";
      description.disabled = true;
      description.className = "dp-description";
      if (segment == "yr") { description.value = "JAAR"; }
      else if (segment == "month") { description.value = "MND"; }
      else if (segment == "day") { description.value = "DAG"; }      
      inst[segment] = description;
      
      // Up Button
      let up = document.createElement("div");
      up.innerHTML = "&#65087;";
      up.className = "dp-up";

      // Current Value
      let val = document.createElement("input");
      val.type = "text";
      val.disabled = true;
      val.className = "dp-val";
      if (segment == "yr") { val.value = "2020"; }
      else if (segment == "month") { val.value = "01"; }
      else if (segment == "day") { val.value = "01"; }      
      inst[segment] = val;

      // Down Button
      let down = document.createElement("div");
      down.innerHTML = "&#65088;";
      down.className = "dp-up";

      // Button click handlers
      up.addEventListener("mousedown", function(){
          dp.spin(id, segment, 1);
      });
      down.addEventListener("mousedown", function(){
          dp.spin(id, segment, 0);
      });
      up.addEventListener("mouseup", dp.sspin);
      up.addEventListener("mouseleave", dp.sspin);
      down.addEventListener("mouseup", dp.sspin);
      down.addEventListener("mouseleave", dp.sspin);

      // Append all the buttons
      box.appendChild(description);
      box.appendChild(up);
      box.appendChild(val);
      box.appendChild(down);
      picker.appendChild(box);
    };
    buttonator("yr");
    buttonator("month");
    buttonator("day");
    
    // (A4) OK BUTTON
    let ok = document.createElement("input");
    ok.type = "button";
    ok.value = "OK";
    ok.className = "dp-ok";
    ok.addEventListener("click", function(){
      dp.set(id);
    });
    picker.appendChild(ok);
  },

  // (B) "HOLD TO SPIN" FOR HOUR + MIN
  stimer : null, // Spin timer
  ssensitive : 100, // lower will spin faster
  spin : function (id, segment, direction) {
    if (dp.stimer == null) {
      dp.sid = id;
      dp.sseg = segment;
      if (segment == "yr") {dp.smax = 2099;}
      else if (segment == "month") {dp.smax = 12;}
      else {dp.smax = 31;}
      //dp.smax = segment == "yr" ? 2099 : 12;
      if (segment == "yr") {dp.smax = 2099;}
      else if (segment == "month") {dp.smax = 12;}
      else {dp.smax = 31;}
      dp.smin = segment == "yr" ? 2020 : 1;
      dp.sdir = direction;
      dp.hmspin();
      dp.stimer = setInterval(dp.hmspin, dp.ssensitive);
    }
  },

  // (C) STOP HR/MIN SPIN
  sspin : function () {
    if (dp.stimer != null) {
      clearInterval(dp.stimer);
      dp.stimer = null;
    }
  },

  // (D) SPIN HR OR MIN
  sid : null, // Instance ID
  sseg : null, // Segment to spin
  smax : null, // Maximum value (23 for yr, 59 for month)
  smin : null, // Minimum value (0 for yr, 0 for month)
  sdir : null, // Spin direction
  hmspin : function () {
    // (D1) CURRENT VALUE
    let box = dp.instances[dp.sid][dp.sseg],
        cv = parseInt(box.value);

    // (D2) SPIN!
    if (dp.sdir) { cv++; }
    else { cv--; }
    if (cv < dp.smin) { cv = dp.smax; }
    if (cv > dp.smax) { cv = dp.smin; }
    if (cv < 10) { cv = "0" + cv; }

    // (D3) UPDATE DISPLAY
    box.value = cv;
  },
  
  // (F) SET SELECTED TIME
  set : function (id) {
    // (F1) GET + FORMAT HH:MM AM/PM
    let inst = dp.instances[id],
    timestamp = dp.instances[id]["yr"].value + "-" +
                dp.instances[id]["month"].value + "-" +
                dp.instances[id]["day"].value;

    // (F2) SET TIMESTAMP
    inst.target.value = timestamp;

    // (F3) CLOSE TIME PICKER (POPUP ONLY)
    if (id==0) {
      inst.wrap.classList.remove("show");
    }
  },

  // (G) ATTACH TIME PICKER TO TARGET
  attach : function (opt) {
  // target - input field
  // wrap - optional, inline time picker

    // (G1) SET INPUT FIELD READONLY
    let target = document.getElementById(opt.target);
    target.readOnly = true;
    
    // (G2) INLINE VERSION - GENERATE TIME PICKER HTML
    if (opt.wrap) {
      dp.init(document.getElementById(opt.wrap), target);
    }
    
    // (G3) POPUP VERSION - SHOW POPUP ON CLICK
    else {
      target.addEventListener("click", function(){
        // Get + set popup time
        let cv = this.value;
        if (cv == "") {
          dp.instances[0].yr.value = "2020";
          dp.instances[0].month.value = "01";
          dp.instances[0].day.value = "01";
        } else {
          dp.instances[0].yr.value = cv.substring(0, 4);
          dp.instances[0].month.value = cv.substring(5, 7);
          dp.instances[0].day.value = cv.substring(8, 10);
        }
        // Set target + show popup
        dp.instances[0].target = target;
        dp.instances[0].wrap.classList.add("show");
      });
    }
  }
};

// (H) CREATE "DEFAULT" POPUP TIME PICKER ON LOAD
window.addEventListener("DOMContentLoaded", function(){
  let pop = document.createElement("div");
  document.body.appendChild(pop);
  pop.id = "dp-pop";
  dp.init(pop);
});
